using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Winnovative.WnvRtfToPdf;

namespace WnvConvertRtfToPdfDemo
{
    public partial class WnvRtfConvertForm : Form
    {
        public WnvRtfConvertForm()
        {
            InitializeComponent();
        }

        private void LoadImageFormats()
        {
            
        }

        private void LoadCompressionLevels()
        {
            string[] pdfCompressionLevels = Enum.GetNames(typeof(PdfCompressionLevel));
            ddlCompression.DataSource = pdfCompressionLevels;
            ddlCompression.SelectedItem = PdfCompressionLevel.Normal.ToString();
        }

        private void LoadPDFFormats()
        {
            string[] pdfFormats = Enum.GetNames(typeof(PdfPageSize));
            ddlPageFormat.DataSource = pdfFormats;
            ddlPageFormat.SelectedItem = PdfPageSize.A4.ToString();
        }

        private void LoadPageOrientations()
        {
            string[] pdfPageOrientations = Enum.GetNames(typeof(PDFPageOrientation));
            ddlPageOrientation.DataSource = pdfPageOrientations;
            ddlPageOrientation.SelectedItem = PDFPageOrientation.Portrait.ToString();
        }

        private void LoadColors()
        {
            string[] colors = Enum.GetNames(typeof(KnownColor));

            ddlHeaderTextColor.DataSource = colors;
            ddlHeaderTextColor.SelectedItem = KnownColor.Black.ToString();

            ddlFooterColor.DataSource = colors;
            ddlFooterColor.SelectedItem = KnownColor.Black.ToString();
        }

        private bool PageIsValid()
        {
            errorProvider.Clear();
            bool invalid = false;

            if (textBoxRTFFilePath.Text.Trim() == String.Empty)
            {
                errorProvider.SetError(textBoxRTFFilePath, "The RTF file path cannot be empty");
                invalid = true;
            }
            if (pnlSettingsPanel.Visible)
            {
                if (radioButtonCustomSize.Checked)
                {
                    // check custom width and height
                    try
                    {
                        int width = int.Parse(textBoxRTFViewerWidth.Text.Trim());
                        if (width < 0)
                            throw new Exception();
                    }
                    catch
                    {
                        errorProvider.SetError(textBoxRTFViewerWidth, "The RTF Viewer width must be a positive integer");
                        invalid = true;
                    }

                    try
                    {
                        int height = int.Parse(textBoxRTFViewerHeight.Text.Trim());
                        if (height < 0)
                            throw new Exception();
                    }
                    catch
                    {
                        errorProvider.SetError(textBoxRTFViewerHeight, "The RTF Viewer height must be a positive integer");
                        invalid = true;
                    }
                }


                try
                {
                    int leftMargin = int.Parse(textBoxLeftMargin.Text.Trim());
                    if (leftMargin < 0)
                        throw new Exception();
                }
                catch
                {
                    errorProvider.SetError(textBoxLeftMargin, "The PDF document left margin must be a positive integer");
                    invalid = true;
                }

                try
                {
                    int rightMargin = int.Parse(textBoxRightMargin.Text.Trim());
                    if (rightMargin < 0)
                        throw new Exception();
                }
                catch
                {
                    errorProvider.SetError(textBoxRightMargin, "The PDF document right margin must be a positive integer");
                    invalid = true;
                }

                try
                {
                    int topMargin = int.Parse(textBoxTopMargin.Text.Trim());
                    if (topMargin < 0)
                        throw new Exception();
                }
                catch
                {
                    errorProvider.SetError(textBoxTopMargin, "The PDF document top margin must be a positive integer");
                    invalid = true;
                }

                try
                {
                    int bottomMargin = int.Parse(textBoxBottomMargin.Text.Trim());
                    if (bottomMargin < 0)
                        throw new Exception();
                }
                catch
                {
                    errorProvider.SetError(textBoxBottomMargin, "The PDF document bottom margin must be a positive integer");
                    invalid = true;
                }

            }
            return !invalid;
        }

        private void lnkBtnSettings_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (!PageIsValid())
                return;

            if (pnlSettingsPanel.Visible)
            {
                pnlSettingsPanel.Visible = false;
                lnkBtnSettings.Text = "More Converter Settings >>";
            }
            else
            {
                pnlSettingsPanel.Visible = true;
                lnkBtnSettings.Text = "<< Hide Settings";
            }
            this.ClientSize = new Size(this.ClientSize.Width, pnlHeader.Height + pnlContent.Height + pnlConvertCommand.Height);
        }

        private void radioButtonAutodetect_CheckedChanged(object sender, EventArgs e)
        {
            pnlRTFViewerSize.Visible = !((RadioButton)sender).Checked;
            this.ClientSize = new Size(this.ClientSize.Width, pnlHeader.Height + pnlContent.Height + pnlConvertCommand.Height);
        }

        private void radioConvertToPDF_CheckedChanged(object sender, EventArgs e)
        {
            pnlPDFSettings.Visible = ((RadioButton)sender).Checked;
            this.ClientSize = new Size(this.ClientSize.Width, pnlHeader.Height + pnlContent.Height + pnlConvertCommand.Height);
        }

        private void cbShowHeader_CheckedChanged(object sender, EventArgs e)
        {
            pnlHeaderOptions.Visible = ((CheckBox)sender).Checked;
            this.ClientSize = new Size(this.ClientSize.Width, pnlHeader.Height + pnlContent.Height + pnlConvertCommand.Height);
        }

        private void cbShowFooter_CheckedChanged(object sender, EventArgs e)
        {
            pnlFooterOptions.Visible = ((CheckBox)sender).Checked;
            this.ClientSize = new Size(this.ClientSize.Width, pnlHeader.Height + pnlContent.Height + pnlConvertCommand.Height);
        }

        private void WnvHtmlConvertForm_Load(object sender, EventArgs e)
        {
            textBoxRTFFilePath.Text = Path.Combine(Application.StartupPath, @"..\..\convert_sample.rtf");

            LoadImageFormats();
            LoadPDFFormats();
            LoadPageOrientations();
            LoadColors();
            LoadCompressionLevels();
        }

        private PdfConverter GetPDFConverter()
        {
            int pageWidth = 0;
            int pageHeight = 0;

            // set common properties

            pageWidth = int.Parse(textBoxRTFViewerWidth.Text.Trim());
            pageHeight = int.Parse(textBoxRTFViewerHeight.Text.Trim());

            // create the PDF converter
            PdfConverter pdfConverter = new PdfConverter(pageWidth, pageHeight);

            pdfConverter.LicenseKey = "Yuz97frt/e354/3t/vzj/P/j9PT09A==";

            // set PDF options
            pdfConverter.PdfDocumentOptions.PdfPageSize = (PdfPageSize)Enum.Parse(typeof(PdfPageSize), ddlPageFormat.SelectedItem.ToString());
            pdfConverter.PdfDocumentOptions.PdfPageOrientation = (PDFPageOrientation)Enum.Parse(typeof(PDFPageOrientation), ddlPageOrientation.SelectedItem.ToString());
            pdfConverter.PdfDocumentOptions.PdfCompressionLevel = (PdfCompressionLevel)Enum.Parse(typeof(PdfCompressionLevel), ddlCompression.SelectedItem.ToString());
            pdfConverter.PdfDocumentOptions.ShowHeader = cbShowHeader.Checked;
            pdfConverter.PdfDocumentOptions.ShowFooter = cbShowFooter.Checked;
            pdfConverter.PdfDocumentOptions.LeftMargin = int.Parse(textBoxLeftMargin.Text.Trim());
            pdfConverter.PdfDocumentOptions.RightMargin = int.Parse(textBoxRightMargin.Text.Trim());
            pdfConverter.PdfDocumentOptions.TopMargin = int.Parse(textBoxTopMargin.Text.Trim());
            pdfConverter.PdfDocumentOptions.BottomMargin = int.Parse(textBoxBottomMargin.Text.Trim());

            pdfConverter.PdfHeaderOptions.HeaderText = textBoxHeaderTitle.Text;
            pdfConverter.PdfHeaderOptions.HeaderTextColor = Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), ddlHeaderTextColor.SelectedItem.ToString()));
            pdfConverter.PdfHeaderOptions.HeaderSubtitleText = textBoxHeadeSubtitle.Text;
            pdfConverter.PdfHeaderOptions.DrawHeaderLine = cbDrawHeaderLine.Checked;

            pdfConverter.PdfFooterOptions.FooterText = textBoxFooterText.Text;
            pdfConverter.PdfFooterOptions.FooterTextColor = Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), ddlFooterColor.SelectedItem.ToString()));
            pdfConverter.PdfFooterOptions.DrawFooterLine = cbDrawFooterLine.Checked;
            pdfConverter.PdfFooterOptions.PageNumberText = textBoxPageNumberText.Text;
            pdfConverter.PdfFooterOptions.ShowPageNumber = cbShowPageNumber.Checked;

            pdfConverter.AvoidTextBreak = cbAvoidTextBreak.Checked;

            return pdfConverter;
        }

        

        private void btnConvert_Click(object sender, EventArgs e)
        {
            if (!PageIsValid())
                return;

            this.Cursor = Cursors.WaitCursor;

            string outFile = null;

            try
            {
                
                
                    PdfConverter pdfConverter = GetPDFConverter();

                    outFile = Path.Combine(Application.StartupPath, "RenderedPage.pdf");
                    try
                    {
                        pdfConverter.SavePdfFromRtfFileToFile(textBoxRTFFilePath.Text, outFile);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                        return;
                    }
                
            }
            finally
            {
                this.Cursor = Cursors.Arrow;
            }

            DialogResult dr = MessageBox.Show("Open the rendered file in an external viewer?", "Open Rendered File", MessageBoxButtons.YesNo);
            if (dr == DialogResult.Yes)
            {
                try
                {
                    System.Diagnostics.Process.Start(outFile);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                    return;
                }
            }
        }

        private void radioButtonConvertToPDFSelectable_CheckedChanged(object sender, EventArgs e)
        {
            pnlPDFSettings.Visible = ((RadioButton)sender).Checked;
            this.ClientSize = new Size(this.ClientSize.Width, pnlHeader.Height + pnlContent.Height + pnlConvertCommand.Height);
        }

        private void linkLabelViewRTF_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            using (RtfViewer rtfViewer = new RtfViewer())
            {
                rtfViewer.RtfFileFullPath = textBoxRTFFilePath.Text;
                rtfViewer.RtfViewerWidth = int.Parse(textBoxRTFViewerWidth.Text.Trim());
                rtfViewer.ShowDialog();
            }
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog fd = new OpenFileDialog())
            {
                fd.Filter = "rtf files (*.rtf)|*.rtf|All files (*.*)|*.*";
                if (fd.ShowDialog() == DialogResult.OK)
                {
                    textBoxRTFFilePath.Text = fd.FileName;
                }
            }

        }

        private void LaunchHomePage()
        {
            try
            {
                System.Diagnostics.Process.Start("http://www.winnovative-software.com?rtf_to_pdf=1");
            }
            catch (System.ComponentModel.Win32Exception)
            {
                MessageBox.Show("Could not open an internet browser.\nPlease visit: " + "http://www.winnovative-software.com?rtf_to_pdf=1");
            }
        }

        private void pictureBoxLogo_Click(object sender, EventArgs e)
        {
            LaunchHomePage();
        }

        private void lnkHomePage_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            LaunchHomePage();
        }

    }
}